#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pyimporters_plugins']

package_data = \
{'': ['*']}

install_requires = \
['pydantic==1.7.3', 'pandas', 'rdflib']

entry_points = \
{'pyimporters.plugins': ['csv = pyimporters_plugins.csv:CSVKnowledgeParser',
                         'skos = pyimporters_plugins.skos:SKOSKnowledgeParser',
                         'txt = pyimporters_plugins.csv:TXTKnowledgeParser']}

setup(name='pyimporters-plugins',
      version='0.1.26',
      description='Sherpa knowledge import plugins',
      author='Olivier Terrier',
      author_email='olivier.terrier@kairntech.com',
      url='https://kairntech.com/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
